/*
 * projet basica
 * 
 */


#include <U8g2lib.h>
#include <U8x8lib.h>

#include "Command.h"
#include "CLI.h"
#include "config.h"


//joystick input
#define JOY_X_INPUT 23
#define JOY_Y_INPUT 22

#define BTN_ANA_1 23
#define BTN_ANA_2 22
#define BTN_ANA_3 21
#define BTN_ANA_4 20


//ledoption
#define LED 13

//lcd initialisation
//U8G2_SSD1306_128X64_NONAME_F_HW_I2C u8g2(U8G2_R0, /* reset=*/ U8X8_PIN_NONE);
//U8G2_SSD1306_128X64_NONAME_F_SW_I2C u8g2(U8G2_R0, /* clock=*/ SCL, /* data=*/ SDA, /* reset=*/ U8X8_PIN_NONE);   // All Boards without Reset of the Display
// Configure the number of buttons.  Be careful not
// to use a pin for both a digital button and analog
// axis.  The pullup resistor will interfere with
// the analog voltage.
const int numButtons = 4;  // 16 for Teensy, 32 for Teensy++

/*
 * lcd_setup (void)
 * 
 * description:
 * initialize lcd display option
 */
void lcd_setup(void) {
  #if 0
  u8g2.setFont(u8g2_font_6x10_tf);
  u8g2.setFontRefHeightExtendedText();
  u8g2.setDrawColor(1);
  u8g2.setFontPosTop();
  u8g2.setFontDirection(0);
  #endif
}

static byte btn_id[numButtons];

void setup() {
 
  // joytickinitialization
  Joystick.useManualSend(true);

  /*
  for (int i=0; i<numButtons; i++) {
    pinMode(i, INPUT_PULLUP);
  }
*/
  // lcd initialization
#if 0
  u8g2.begin();
  u8g2.clearBuffer ();
  lcd_setup();

  u8g2.drawStr(48, 32, "Start");
  u8g2.sendBuffer ();
#endif
  // intialize command
  cli_open ();
  
  // initialize options
  load_config ();

  //led init
  // initialize led
  pinMode(LED, OUTPUT);
  digitalWrite (LED, LOW);

  //delay (50);

  // buttoninitialisation
  btn_id[0] = BTN_ANA_1;
  btn_id[1] = BTN_ANA_2;
  btn_id[2] = BTN_ANA_3;
  btn_id[3] = BTN_ANA_4;
  
}

//current button state
static byte allButtons[numButtons];

//previous buttonstate
static byte prevButtons[numButtons];

// angle
static int angle=0;

// timing info
static long lastloop = 0;

static boolean anyChange = false;
static int XC = 0;
static int YC =0;
static int X;
static int Y;
static int cnt = 3;

/*
 * lcd_display (int X, int Y)
 * 
 * description:
 * display gamepad info
 * 
 * input:
 * X current joystick position X
 * Y current joystick position Y
 * XC joystick orrected position X
 * YC joystick correted position Y
 */
 void lcd_display (void)
 { 

  Serial.begin (9600);
  
  #if 0
  static char buf[64] = "XX";
  
  // display info
  u8g2.clearBuffer ();

   // joystick position
   
  snprintf (buf,sizeof(buf), "%04d %04d", X, XC);
  u8g2.drawStr(0, 16, buf);
  snprintf (buf,sizeof(buf), "%04d %04d", Y, YC);
  u8g2.drawStr(64, 16, buf);
  u8g2.drawDisc(X*128/1024,Y*64/1024,2);
  u8g2.drawCircle(XC*128/1024,YC*64/1024,3);
  //u8g2.drawLine (64,32,X*128/1024,Y*64/1024);
  //u8g2.drawLine (64,32,XC*128/1024,YC*64/1024);
  // button state
  for (int i = 0;i < numButtons;i++)
  {
    u8g2.drawStr(i*16, 32, allButtons[i] ==0 ? "_" : "X");
  }

  // time
  snprintf (buf,sizeof(buf), "%02lu ms - %03lu ups", millis () - lastloop,1000/(millis () - lastloop));
  u8g2.drawStr(0, 48, buf);

  // display
  u8g2.sendBuffer ();

  #endif
 }
/*
 * compute_position (int pos)
 * 
 * description:
 * compute realposition according to multiplier
 * 
 * input:
 * pos  Joystick real position
 * mul  multiplier*
 * 
 * return:
 * the function return thecorrected position
 */
 int compute_position (int pos, int mul)
 {
    int rc = (pos - 512) * mul;

    rc+= 512;
    if(rc < 0) rc = 0;
    else if (rc > 1023)
    rc = 1023;

    return (rc);
 }


/*
 * main loop
 */
void loop() {
  
  X = 0; //analogRead(JOY_X_INPUT);
  Y = 0; //analogRead(JOY_Y_INPUT);
  anyChange = false;
  XC = 0;
  YC =0;


  // compute position with multiplier 
  XC = compute_position (X,get_mulx());
  YC = compute_position (Y,get_muly());
  
  // read 6 analog inputs and use them for the joystick axis
  //Joystick.X(XC);
  //Joystick.Y(YC);
  //Joystick.Z(analogRead(2));
  //Joystick.Zrotate(analogRead(3));
  //Joystick.sliderLeft(analogRead(4));
  //Joystick.sliderRight(analogRead(5));
  
  // read digital pins and use them for the buttons
  #if 0
  for (int i=0; i<numButtons; i++) {
    if (digitalRead(i)) {
      // when a pin reads high, the button is not pressed
      // the pullup resistor creates the "on" signal
      allButtons[i] = 0;
    } else {
      // when a pin reads low, the button is connecting to ground.
      allButtons[i] = 1;
    }
    Joystick.button(i + 1, allButtons[i]);
  }
#else
  digitalWrite (LED, LOW);
  for(int i = 0; i < numButtons; i++)
  {
    int val = analogRead (btn_id[i]);
    allButtons[i] = val < 330 ? 1 : 0;
    Joystick.button(i + 1, allButtons[i]);
    if (val < 330)
    {
      digitalWrite (LED, HIGH);  
    }

    Serial.print ("btn");
    Serial.print (i);
    Serial.print (" ");
    Serial.println (val);
  }
#endif

  // make the hat switch automatically move in a circle
  //angle = angle + 1;
  if (angle >= 360) angle = 0;
  Joystick.hat(angle);
  
  // send joystick position and button to Host
  Joystick.send_now();
  
  // display info
  //XC=YC=512;
  lcd_display ();
  
  //// a brief delay, so this runs "only" 200 times per second
  //delay(5);

  // blink led
  if (angle % 18 == 0)
  {
    cnt = 3; 
  }
  
  //digitalWrite (LED, (cnt == 3 || cnt==0) ? HIGH:LOW);
  cnt--;
  
  // process command
  CLI.process();
   
  //update time
  lastloop = millis ();
}

